@@ -6,7 +6,10 @@ from requests import request as _req
 
 from .filesHandler import find_ndb_files
 from .errors import *
+from .popHandler import popDN
 from . import UTypes
+from .Files import Files
+from .algo import _getAlgo
 
 schema = {
    '__docs': []
@@ -24,9 +27,6 @@ def checkTypes(l:list, type):
    '''
    return not list(filter(lambda e:not isinstance(e, type), l))
 
-def _getAlgo(documents, _filter):
-   return [d for d in documents if sum(1 for k, v in d.items() if _filter.get(k)==v) >= len(_filter)]
-
 def get_password():   
    password = _hashpw(_getpass('Password: ').encode('utf-8'), _gensalt())
    return password
@@ -51,7 +51,38 @@ def create_db(filename:str, _password=None):
             file.write(pyonr.dumps(schema))
 
 class NotDBClient:
-   def __init__(self, host:str=None, password=None):
+   '''
+   NotDB Databases client
+
+   >>> NotDBClient('test.ndb', password=password)
+   
+   Full documentation:
+   - [NotDB](https://github.com/nawafalqari/NotDB#readme)
+   - [NotDB Cloud](https://github.com/nawafalqari/NotDB_Cloud#readme)
+   '''
+   def __init__(
+      self,
+      host:str=None,
+      password:str=None):
+      '''
+      NotDB Databases client
+
+      >>> NotDBClient('test.ndb', password=password)
+      
+      Full documentation:
+      - [NotDB](https://github.com/nawafalqari/NotDB#readme)
+      - [NotDB Cloud](https://github.com/nawafalqari/NotDB_Cloud#readme)
+
+      `Parameters:`
+
+      `host`: (optional) database filename/url/ip address,
+      if `None` was given it will scan the directory for one ndb file
+      if there was multiple databases it will raise `InvalidHostError`
+
+      `password`: (optional for unsecured DBs) database's password,
+      must be `str` or `bytes`\n
+      raise `WrongPasswordError` if the password was wrong
+      '''
       if not host:
          self.__host = find_ndb_files('.')
          self.__hostType = 'local'
@@ -97,7 +128,7 @@ class NotDBClient:
             elif isinstance(password, bytes) and not _checkpw(password, self.__CDBData('__password')):
                raise WrongPasswordError()
       
-   # file data
+   # database getters
    @property
    def host(self):
       h = self.__host
@@ -128,6 +159,11 @@ class NotDBClient:
    def hostType(self):
       return self.__hostType
 
+   # Files class
+   @property
+   def files(self) -> Files:
+      return Files(self, self.__read, self.__CRead, self.__CWrite)
+
    # cloud dbs functions
    def __CRead(self):
       return pyonr.loads(_req('BRING', self.__host).content.decode('utf-8'))
@@ -207,7 +243,7 @@ class NotDBClient:
          _dbd = _r.readfile # db data
          
          _dbd['__docs'].append(document)
-         self.__read.write(_dbd)
+         _r.write(_dbd)
 
          return True
       # server db
@@ -281,6 +317,7 @@ class NotDBClient:
 
          return True
       
+      # Cloud db
       _fd = self.__CRead()
       _docs = _fd['__docs']
       full_doc = self.getOne(_filter)
@@ -391,14 +428,47 @@ class NotDBClient:
 
          return True
 
-      raise TypeError(f'"{type}": Invalid type, expecting "notdb.SET" or "notdb.UNSET"')
+      raise TypeError(f'"{type}": Invalid type, check dir(notdb.UTypes)')
 
-   def updateMany(self, _filter:dict, update:dict):
-      pass
+   def updateOnePOP(self, _filter:dict, element:str, element_index:int=-1):
+      '''
+      Remove and return a list item at index
+      (This update type is only for pop)
+
+      >>> db.updateOnePOP(
+      {'name': 'Nawaf'},         <-- filter
+      'skills.programmingLangs', <-- path to array (dot notation)
+      0)                         <-- index (default last)
+      '''
+
+      _fullDoc = self.getOne(_filter)
+      _r = None
+      _fd = None
+      _docs = None
+      if self.hostType == 'local':
+         _r = self.__read
+         _fd = _r.readfile
+         _docs = _fd['__docs']
+      if self.hostType == 'cloud':
+         _fd = self.__CRead()
+         _docs = _fd['__docs']
+      i = _docs.index(_fullDoc)
+      
+      poppedItem = popDN(_docs[i], element, element_index)
+
+      if self.hostType == 'local':
+         _r.write(_fd)
+      if self.hostType == 'cloud':
+         self.__CWrite(_fd)
+
+      return poppedItem
+
+   def __repr__(self):
+      return f'{self.__class__.__name__}(host="{self.host}")'
 
 class NotDBCloudClient(NotDBClient):
    '''
-   **NotDB** Databases On Cloud
+   NotDB Databases On Cloud
 
    >>> NotDBCloudClient('https://example.com/t.ndb', password=password)
    
